function GetDayOfWeek(Year, Month, Day: Integer): Integer;
var
  Century, yr, dw: Integer;
begin
  if Month < 3 then
  begin
    Inc(Month, 10);
    Dec(Year);
  end else Dec(Month, 2);
  Century := Year div 100;
  yr := year mod 100;
  dw := (((26 * month - 2) div 10) + day + yr + (yr div 4) +
        (Century div 4) - (2 * Century)) mod 7;
  if dw < 0 then Result := dw + 7
  else Result := dw;
end;

function JulianDay(AYear, AMonth, ADay: Integer): Integer;
var
  Tmp1, Tmp2, Tmp3, Tmp4: Integer;
begin
  if AMonth > 2 then
  begin
    Tmp1 := AMonth - 3;
    Tmp2 := AYear;
  end
 else
  begin
    Tmp1 := AMonth + 9;
    Tmp2 := AYear - 1;
  end;
  Tmp1 := (Tmp1 * 153) + 2;
  Tmp3 := (Tmp2 div 100) * 146097;
  Tmp4 := (Tmp2 mod 100) * 1461;
  Result := (Tmp1 div 5) + (Tmp3 div 4) + (Tmp4 div 4) + ADay + 1721119;
end;

procedure JulianDayToDate(JDay: Integer; var AYear, AMonth, ADay: Integer);
var
  Tmp1, Tmp2, Tmp3: Integer;
begin
  Tmp1 := JDay - 1721119;
  Tmp3 := (4 * Tmp1 - 1) div 146097;
  Tmp1 := (4 * Tmp1 - 1) mod 146097;
  Tmp2 := Tmp1 div 4;
  Tmp1 := (4 * Tmp2 + 3) div 1461;
  Tmp2 := (4 * Tmp2 + 3) mod 1461;
  Tmp2 := (Tmp2 + 4) div 4;
  Amonth := (5 * Tmp2 - 3) div 153;
  Tmp2 := (5 * Tmp2 - 3) mod 153;
  ADay := ((Tmp2 + 5) div 5);
  AYear := (100 * Tmp3 + Tmp1);
  if AMonth < 10 then AMonth := AMonth + 3
 else
  begin
    AMonth := AMonth - 9;
    AYear := AYear + 1;
  end;
end;

function IsValidDate(AYear, AMonth, ADay: Integer): Boolean;
var
  JulDay: Integer;
  ycal, mcal, dcal : Integer;
begin
  JulDay := JulianDay(AYear, AMonth, ADay);
  JulianDayToDate(JulDay, ycal, mcal, dcal);
  Result := (AYear = ycal) and (AMonth = mcal) and (ADay = dcal)
end;

function WeekOfYear(Y, M, D: Integer): Integer;
var
  FirstJulian, JulDay: Integer;
  FirstDay: Integer;
begin
  JulDay := JulianDay(Y, M, D);
  FirstJulian := JulianDay(Y, 1, 1);
  FirstDay := GetDayOfWeek(Y, 1, 1);
  if FirstDay <> 0 then Inc(FirstJulian, 7-Ord(FirstDay));
  if JulDay < FirstJulian then Result := 0
 else Result := (JulDay - FirstJulian + 7) div 7;
end;

function IsLeapYear(AYear: Integer): Boolean;
begin
  Result := (AYear mod 4 = 0) and ((AYear mod 100 <> 0) or (AYear mod 400 = 0));
end;

function DaysPerMonth(AYear, AMonth: Integer): Integer;
const
  DaysInMonth: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
begin
  Result := DaysInMonth[AMonth];
  if (AMonth = 2) and IsLeapYear(AYear) then Inc(Result);
end;